package net.wanji.utc.hisense.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.ScheduleUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.mapper.VIntersectionMapper;
import net.wanji.utc.hisense.mapper.VNtcipTimeBaseScheduleMapper;
import net.wanji.utc.hisense.pojo.result.CoordinationStatus;
import net.wanji.utc.hisense.pojo.result.CrossSchemeRings;
import net.wanji.utc.hisense.pojo.view.VIntersectionEntity;
import net.wanji.utc.hisense.pojo.view.VNtcipTimeBaseSchedule;
import net.wanji.utc.hisense.service.StaticInfoService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:29
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class StaticInfoServiceImpl implements StaticInfoService {

    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;
    @Resource
    private VIntersectionMapper vIntersectionMapper;
    @Resource
    private VNtcipTimeBaseScheduleMapper vNtcipTimeBaseScheduleMapper;

    @Override
    public List<CrossInfoPO> crossBasicInfo(CrossInfoDTO crossInfoDTO) throws Exception {
        Integer id = manufacturerInfoMapper.selectIdByCode(crossInfoDTO.getManufacturerCode());
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(id);
        List<String> codeList = crossInfoPOList.stream().map(CrossInfoPO::getCode).collect(Collectors.toList());
        Map<String, String> codeCrossIdMap = crossInfoPOList.stream().collect(Collectors.toMap(CrossInfoPO::getCode, CrossInfoPO::getId));
        List<CrossInfoPO> crossInfoPOS = new ArrayList<>();
        if (StringUtils.equalsIgnoreCase("HISENSE", crossInfoDTO.getManufacturerCode())) {
            LambdaQueryWrapper<VIntersectionEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(VIntersectionEntity::getCIntsID, codeList);
            List<VIntersectionEntity> vIntersectionEntities = vIntersectionMapper.selectList(queryWrapper);
            for (VIntersectionEntity vIntersectionEntity : vIntersectionEntities) {
                String cIntsID = vIntersectionEntity.getCIntsID();
                CrossInfoPO crossInfoPO = new CrossInfoPO();
                crossInfoPO.setId(codeCrossIdMap.get(cIntsID));
                crossInfoPO.setCode(cIntsID);
                crossInfoPO.setName(vIntersectionEntity.getCIntsName());
                crossInfoPO.setIp(vIntersectionEntity.getCCONTROLLERIP());
                String[] array = {String.valueOf(vIntersectionEntity.getLongitude()), String.valueOf(vIntersectionEntity.getLatitude())};
                crossInfoPO.setLocation(StringUtils.join(array, ","));
                crossInfoPO.setVersion(vIntersectionEntity.getCControllerModel());
                crossInfoPO.setManufacturerId(22);
                crossInfoPOS.add(crossInfoPO);
            }
        }
        return crossInfoPOS;
    }

    @Override
    public SchemePhaseLightsVO schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        return null;
    }

    @Override
    public List<PlanSectionVO> planSection(PlanSectionDTO planSectionDTO) throws Exception {
        return null;
    }

    @Override
    public List<CrossSchedulesPO> crossSchedules(CrossSchedulesDTO crossSchedulesDTO) throws Exception {

        Map<String, String> codeCrossIdMap = getCodeCrossIdMap(crossSchedulesDTO);

        // 路口编号列表
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        // 信号机编号
        List<String> crossCodeList = crossInfoMapper.selectCrossCodesByIds(crossIdList);
        List<CrossSchedulesPO> results = new ArrayList<>(crossCodeList.size());
        if (!CollectionUtils.isEmpty(crossCodeList)) {
            for (String code : crossCodeList) {
                LambdaQueryWrapper<VNtcipTimeBaseSchedule> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(VNtcipTimeBaseSchedule::getCIntsID, code);
                List<VNtcipTimeBaseSchedule> vNtcipTimeBaseSchedules = vNtcipTimeBaseScheduleMapper.selectList(queryWrapper);
                if (!CollectionUtils.isEmpty(vNtcipTimeBaseSchedules)) {
                    for (VNtcipTimeBaseSchedule vNtcipTimeBaseSchedule : vNtcipTimeBaseSchedules) {
                        if (Objects.nonNull(vNtcipTimeBaseSchedule)) {
                            CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                            crossSchedulesPO.setCrossId(codeCrossIdMap.get(code));
                            crossSchedulesPO.setScheduleNo(vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                            crossSchedulesPO.setName("海信调度" + vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                            // 海信月份
                            Integer hisenseMonth = vNtcipTimeBaseSchedule.getNTimeBaseScheduleMonth();
                            // 星期
                            Integer hisenseWeek = vNtcipTimeBaseSchedule.getNTimeBaseScheduleDay();
                            // 日期 2147483647
                            Integer hisenseDays = vNtcipTimeBaseSchedule.getNTimeBaseScheduleDate();
                            String monthsStr = ScheduleUtil.getSchedule(hisenseMonth.longValue(), 1);
                            String daysStr = ScheduleUtil.getSchedule(hisenseDays.longValue(), 2);
                            String weekStr = ScheduleUtil.getSchedule(hisenseWeek.longValue(), 3);
                            if (StringUtils.isNotBlank(weekStr)) {
                                String[] weekSplit = weekStr.split(",");
                                for (String oneWeek : weekSplit) {
                                    CrossSchedulesPO crossSchedulesPOWeek = new CrossSchedulesPO();
                                    crossSchedulesPOWeek.setCrossId(codeCrossIdMap.get(code));
                                    crossSchedulesPOWeek.setScheduleNo(vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                                    crossSchedulesPOWeek.setName("海信调度" + vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                                    crossSchedulesPOWeek.setWeek(Integer.valueOf(oneWeek));
                                    crossSchedulesPOWeek.setMonths(monthsStr);
                                    crossSchedulesPOWeek.setDays(daysStr);
                                    crossSchedulesPOWeek.setPlanId(0);
                                    results.add(crossSchedulesPOWeek);
                                }
                            } else {
                                // 不执行周调度
                                CrossSchedulesPO crossSchedulesPOMonthDays = new CrossSchedulesPO();
                                crossSchedulesPOMonthDays.setCrossId(codeCrossIdMap.get(code));
                                crossSchedulesPOMonthDays.setScheduleNo(vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                                crossSchedulesPOMonthDays.setName("海信调度" + vNtcipTimeBaseSchedule.getNTimeBaseScheduleNumber());
                                crossSchedulesPOMonthDays.setWeek(null);
                                crossSchedulesPOMonthDays.setMonths(monthsStr);
                                crossSchedulesPOMonthDays.setDays(daysStr);
                                crossSchedulesPOMonthDays.setPlanId(0);
                                results.add(crossSchedulesPOMonthDays);
                            }
                        }
                    }
                }
            }
        }
        return results;
    }

    /**
     * 通过厂商编码，获取map<code 信号机编码, crossId 路口编号></>
     *
     * @param crossSchedulesDTO
     * @return
     */
    private Map<String, String> getCodeCrossIdMap(CrossSchedulesDTO crossSchedulesDTO) {
        String manufacturerCode = crossSchedulesDTO.getManufacturerCode();
        Integer id = manufacturerInfoMapper.selectIdByCode(manufacturerCode);
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(id);
        List<String> codeList = crossInfoPOList.stream().map(CrossInfoPO::getCode).collect(Collectors.toList());
        Map<String, String> codeCrossIdMap = crossInfoPOList.stream().collect(Collectors.toMap(CrossInfoPO::getCode, CrossInfoPO::getId));
        return codeCrossIdMap;
    }

    @Override
    public CoordinationStatus crossPlan(String crossId) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossId);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_12);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.7. 协调状态 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String spot = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Spot");
            String CoordPatternStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordPatternStatus");
            String CoordCycleStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordCycleStatus");
            String CoordSyncStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordSyncStatus");
            CoordinationStatus coordinationStatus = CoordinationStatus.builder()
                    .spot(crossId)
                    .coordPatternStatus(CoordPatternStatus)
                    .coordCycleStatus(CoordCycleStatus)
                    .coordSyncStatus(CoordSyncStatus)
                    .build();
            return coordinationStatus;

        } else {
            //失败
            log.error("请求 5.7. 协调状态 失败");
        }

        return null;
    }

    @Override
    public CrossSchemeRings crossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossSchemeRingsDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_30);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText(crossSchemeRingsDTO.getPattern());
        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.15.根据方案号获取配时方案环图 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String patternRest = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Pattern");
            String cycleLen = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CycleLen");
            String ringCount = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("RingCount");
            String offset = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Offset");
            List<Element> cycleElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/CycleList");
            List<Element> phaseElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/PhaseList");
            CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
            crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
            crossSchemeRings.setPattern(patternRest);
            crossSchemeRings.setRingCount(ringCount);
            crossSchemeRings.setCycleLen(cycleLen);
            crossSchemeRings.setOffset(offset);

            JSONObject jsonObject = new JSONObject();
            for (Element cycle : cycleElementList) {
                List<Node> elements = cycle.elements();
                for (Node node : elements) {
                    node.getName();
                    jsonObject.put(node.getName(), node.getText());
                }
            }
            List<CrossSchemeRings.Phase> phaseList = new ArrayList<>();
            for (Element el : phaseElementList) {
                CrossSchemeRings.Phase phase = new CrossSchemeRings.Phase();
                phase.setRedTime(el.valueOf("RedTime"));
                phase.setYellowTime(el.valueOf("YellowTime"));
                phase.setGreenTime(el.valueOf("GreenTime"));
                phase.setDirection(el.valueOf("Direction"));
                phase.setMinGTime(el.valueOf("MinGTime"));
                phase.setMaxGTime(el.valueOf("MaxGTime"));
                phase.setChannelDim(el.valueOf("ChannelDim"));
                phase.setConcurrent(el.valueOf("Concurrent"));
                phaseList.add(phase);
            }
            crossSchemeRings.setCycleList(jsonObject);
            crossSchemeRings.setPhaseList(phaseList);
            return crossSchemeRings;

        } else {
            //失败
            log.error("请求 5.15. 方案环图请求 失败");
        }
        return null;
    }
}
